package winterwell.utils.containers;

import java.util.AbstractList;
import java.util.List;

/**
 * An integer range. Can be iterated over, e.g.
 * <code>for(int i : new IntRange(10,20)) ...</code>
 * 
 * @see Range
 * @author Daniel
 * 
 */
public final class IntRange extends AbstractList<Integer> implements
		List<Integer> {

	public final int high;

	public final int low;

	/**
	 * Create an IntRange [low, high] - inclusive of end points
	 * 
	 * @param a
	 * @param b
	 *            The order of a,b is not important
	 */
	public IntRange(int a, int b) {
		if (a < b) {
			low = a;
			high = b;
		} else {
			low = b;
			high = a;
		}
	}

	/**
	 * Is x inside this range? Includes the end values.
	 */
	public boolean contains(int x) {
		return x >= low && x <= high;
	}

	@Override
	public Integer get(int index) {
		assert contains(index + low);
		return index + low;
	}

	@Override
	public int size() {
		return high - low + 1;
	}

	@Override
	public String toString() {
		return "[" + low + ", " + high + "]";
	}
}
