package winterwell.utils.io;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * Annotation for use with {@link ArgsParser}. Add this to fields in the
 * settings object.
 */
@Retention(RetentionPolicy.RUNTIME)
// Meta-annotation for "Don't throw this away during compilation"
@Target({ ElementType.FIELD })
// Meta-annotation for "Only allowed on fields"
public @interface Option {
	/**
	 * Displayed in the default usage message.
	 * 
	 * @see ArgsParser#getOptionsMessage()
	 */
	String description() default "";

	/**
	 * If true, an exception will be generated by {@link ArgsParser} if this
	 * option is not set.
	 */
	boolean required() default false;

	/**
	 * Comma separated list of tokens which can be used to set this parameter.
	 * Note: These must begin with a "-"
	 * <p>
	 * E.g. @Option(tokens = "-v,-verbose")
	 * <p>
	 * If unset, the field-name is used, e.g. "-myField"
	 */
	String tokens() default "";
}