package winterwell.utils.reporting;

import java.io.PrintWriter;
import java.io.Serializable;
import java.io.StringWriter;
import java.util.logging.Level;

import winterwell.utils.Environment;
import winterwell.utils.Printer;
import winterwell.utils.time.Time;

/**
 * A log report (just a simple time + message + level).
 * 
 * @author daniel
 * 
 */
public class Report implements Serializable {

	private static final long serialVersionUID = 1L;
	private String details = "";
	private final Level level;
	private final String msg;
	public Object ref;

	private final String tag;
	private final Time time = new Time();

	public Report(String tag, Exception ex) {
		this.tag = tag;
		ref = ex;
		msg = ex.getMessage();
		level = Level.SEVERE;
		StringWriter sw = new StringWriter();
		PrintWriter w = new PrintWriter(sw);
		ex.printStackTrace(w);
		details = sw.toString();
	}

	public Report(String tag, String msg, Level level) {
		this.tag = tag;
		this.msg = msg;
		this.level = level;
	}

	public String getMessage() {
		return msg;
	}

	public Time getTime() {
		return time;
	}

	/**
	 * [time] level message details context
	 */
	@Override
	public String toString() {
		return Environment.get().get(Printer.INDENT)
				+ Printer.format("[{0}]	{1}	#{2}	{3}	{4}	{5}", time, level,
						tag, msg, details, Log.getContextMessage());
	}

}
