package winterwell.utils.time;

import java.util.Calendar;

import winterwell.utils.containers.AbstractIterator;

/**
 * Iterate through a range of dates.
 * 
 * @testedby {@link TimeIteratorTest}
 * @author Daniel
 */
public final class TimeIterator implements Iterable<Time> {

	final Calendar cal;

	private final Time end;
	private final Dt step;

	/**
	 * Create a new TimeIterator with the given stepping. Stepping uses the
	 * local Calendar, so stepping in months works as you'd want (e.g. you can
	 * construct an iterator for the 1st of every month).
	 * 
	 * @param range
	 *            start time and end time, inclusive (end time may not be
	 *            included if adding step*dt to start time oversteps it).
	 */
	public TimeIterator(Time start, Time end, Dt step) {
		assert !start.isAfter(end) : start + " is after " + end;
		cal = start.getCalendar();
		this.end = end;
		this.step = step;
	}

	public Dt getStep() {
		return step;
	}

	@Override
	public AbstractIterator<Time> iterator() {
		final Calendar cal2 = (Calendar) cal.clone();
		return new AbstractIterator<Time>() {
			@Override
			public Time next2() {
				Time now = new Time(cal2);
				if (now.isAfter(end))
					return null;
				step.addTo(cal2);
				return now;
			}
		};
	}

	@Override
	public String toString() {
		return "TimeIterator [now=" + cal + ", end=" + end + ", step=" + step
				+ "]";
	}

}
